package org.bdware.sc.registry;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.sc.util.ExceptionUtil;

import javax.json.Json;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class DBConnector {
  private static final Logger LOGGER = LogManager.getLogger(DBConnector.class);
  public static Connection jdbcConnection;
  public static String url;
  public static String userName;
  public static String password;


  public static void init() throws SQLException {
    try {
      JsonObject ret = new JsonObject();
      ret.addProperty("success", "yes");
      System.out.println(ret.get("success"));
      Class.forName("org.postgresql.Driver");
      connClose(jdbcConnection);
    } catch (Exception e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    jdbcConnection = DriverManager.getConnection(url, userName, password);
  }

  public static void connect() throws SQLException {
    if (url == null) init();
    if (jdbcConnection == null || jdbcConnection.isClosed())
      jdbcConnection = DriverManager.getConnection(url, userName, password);
  }

  private static void connClose(Connection conn) {
    if (conn == null) return;
    try {
      if (conn != null && !conn.isClosed()) conn.close();
    } catch (Exception e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
  }

  private static void pstmtClose(PreparedStatement pstmt) {
    if (pstmt == null) return;
    try {
      if (pstmt != null && !pstmt.isClosed()) pstmt.close();
    } catch (Exception e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
  }

  public static void insert(String schema, String table, Map<String, Object> items) {
    // prepare columns and values needed in sql
    StringBuilder columns = new StringBuilder();
    StringBuilder values = new StringBuilder();

    List<Map.Entry<String, Object>> list = new ArrayList<>(items.entrySet());
    list.forEach(item -> {
      columns.append("\"").append(item.getKey()).append("\",");
      values.append("?,");
    });
    columns.deleteCharAt(columns.lastIndexOf(","));
    values.deleteCharAt(values.lastIndexOf(","));

    String sql = "INSERT INTO \"" + schema + "\".\"" + table + "\" (" + columns + ") VALUES (" + values + ")";

    // execute sql
    PreparedStatement pstmt = null;
    try {
      connect();
      pstmt = jdbcConnection.prepareStatement(sql);
      int i = 1;
      for (Map.Entry<String, Object> item : list) {
        pstmt.setObject(i, item.getValue());
        i++;
      }
      System.out.println(pstmt);
      pstmt.executeUpdate();
    } catch (SQLException e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    pstmtClose(pstmt);
    connClose(jdbcConnection);
  }

  public static String querySampleByMetaID(String doid) {
    String ret = "";
    String sql = "SELECT sample FROM registry.meta_data WHERE doid = ?;";

    // execute sql
    PreparedStatement pstmt = null;
    try {
      connect();
      pstmt = jdbcConnection.prepareStatement(sql);
      pstmt.setString(1, doid);
      ResultSet rs;
      rs = pstmt.executeQuery();
      if (rs.next()) {
        ret = rs.getString("sample");
      }
      pstmt.execute();
    } catch (SQLException e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    pstmtClose(pstmt);
    connClose(jdbcConnection);
    return ret;
  }

  public static String queryInfoByMetaID(String doid) {
    String ret = "";
    String sql = "SELECT description FROM registry.meta_data WHERE doid = ?;";

    // execute sql
    PreparedStatement pstmt = null;
    try {
      connect();
      pstmt = jdbcConnection.prepareStatement(sql);
      pstmt.setString(1, doid);
      ResultSet rs;
      rs = pstmt.executeQuery();
      if (rs.next()) {
        ret = rs.getString("description");
      }
      pstmt.execute();
    } catch (SQLException e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    pstmtClose(pstmt);
    connClose(jdbcConnection);
    return ret;
  }

  public static JsonObject queryMetaStandardByDate(Timestamp startTime, Timestamp endTime, int offset, int count) {
    String sql = "select * from registry.meta_data where \"createTime\" >= ? and \"createTime\" <= ? limit ? offset ? ;";

    // execute sql
    PreparedStatement pstmt = null;
    try {
      connect();
      pstmt = jdbcConnection.prepareStatement(sql);
      pstmt.setInt(3, count);
      pstmt.setInt(4, offset);
      pstmt.setObject(1, startTime);
      pstmt.setObject(2, endTime);

      String[] columns = {
              "name",
              "createTime",
              "updateTime",
              "creator",
              "updater",
              "registryID",
              "controlType",
              "status",
              "version",
              "doid",
              "description",
              "fields"
      };

      System.out.println(pstmt);

      ResultSet rs = null;
      int total = 0;
      JsonObject ret = new JsonObject();
      JsonArray data = new JsonArray();
      rs = pstmt.executeQuery();
      while (rs.next()) {
        total ++;
        JsonObject row = new JsonObject();
        for (String col : columns) {
          if (col.equals("createTime") || col.equals("updateTime"))
            row.addProperty(col, rs.getTimestamp(col).toString());
          else if (col.equals("status"))
            row.addProperty(col, rs.getBoolean(col));
          else
            row.addProperty(col, rs.getString(col));
        }
        data.add(row);
      }
      ret.addProperty("total", total);
      ret.add("data", data);
      rs.close();

      pstmtClose(pstmt);
      connClose(jdbcConnection);

      return ret;
    } catch (SQLException e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    return null;
  }



  public static JsonObject queryMapRuleByOffset(int offset, int count, String localDoId) {
    String sql = "select * from registry.map_rule where \"localMetadata\" = ? limit ? offset ? ;";

    // execute sql
    PreparedStatement pstmt = null;
    try {
      connect();
      pstmt = jdbcConnection.prepareStatement(sql);
      pstmt.setString(1, localDoId);
      pstmt.setInt(2, count);
      pstmt.setInt(3, offset);

      String[] columns = {
              "name",
              "localMetadata",
              "remoteMetadata",
              "mapRule",
              "version",
              "createTime",
              "updateTime",
              "creator",
              "updater",
              "status"
      };

      System.out.println(pstmt);

      ResultSet rs = null;
      int total = 0;
      JsonObject ret = new JsonObject();
      JsonArray data = new JsonArray();
      rs = pstmt.executeQuery();
      while (rs.next()) {
        total ++;
        JsonObject row = new JsonObject();
        for (String col : columns) {
          if (col.equals("createTime") || col.equals("updateTime"))
            row.addProperty(col, rs.getTimestamp(col).toString());
          else if (col.equals("status"))
            row.addProperty(col, rs.getBoolean(col));
          else
            row.addProperty(col, rs.getString(col));
        }
        data.add(row);
      }
      ret.addProperty("total", total);
      ret.add("data", data);
      rs.close();

      pstmtClose(pstmt);
      connClose(jdbcConnection);

      return ret;
    } catch (SQLException e) {
      LOGGER.error(e.getMessage());
      LOGGER.debug(ExceptionUtil.exceptionToString(e));
    }
    return null;
  }
}
